@@ -52,6 +52,9 @@
 /*	VSTREAM *stream;
 /*	char	*format;
 /*	va_list	ap;
+/*
+/*	int	smtp_forbid_bare_lf;
+/*	int	smtp_seen_bare_lf;
 /* AUXILIARY API
 /*	int	smtp_get_noexcept(vp, stream, maxlen, flags)
 /*	VSTRING	*vp;
@@ -131,11 +134,17 @@
 /*	smtp_vprintf() is the machine underneath smtp_printf().
 /*
 /*	smtp_get_noexcept() implements the subset of smtp_get()
-/*	without timeouts and without making long jumps. Instead,
+/*	without timeouts and without making long jumps. Instead
 /*	query the stream status with vstream_feof() etc.
+/*	This function will set smtp_forbid_bare_lf when flagging
+/*	input with a bare newline byte.
 /*
 /*	smtp_timeout_setup() is a backwards-compatibility interface
 /*	for programs that don't require deadline or data-rate support.
+/*
+/*	smtp_forbid_bare_lf controls whether smtp_get_noexcept()
+/*	will set smtp_seen_bare_lf when the line that was read last
+/*	ended with a bare newline byte.
 /* DIAGNOSTICS
 /* .fi
 /* .ad
@@ -214,6 +223,8 @@
   * the buffer. Such system calls would really hurt when receiving or sending
   * body content one line at a time.
   */
+int     smtp_forbid_bare_lf;
+int     smtp_seen_bare_lf;
 
 /* smtp_timeout_reset - reset per-stream error flags */
 
@@ -376,6 +387,8 @@ int     smtp_get_noexcept(VSTRING *vp, VSTREAM *stream, ssize_t bound, int flags
     int     last_char;
     int     next_char;
 
+    smtp_seen_bare_lf = 0;
+
     /*
      * It's painful to do I/O with records that may span multiple buffers.
      * Allow for partial long lines (we will read the remainder later) and
@@ -418,6 +431,9 @@ int     smtp_get_noexcept(VSTRING *vp, VSTREAM *stream, ssize_t bound, int flags
 	 */
     case '\n':
 	vstring_truncate(vp, VSTRING_LEN(vp) - 1);
+	if (smtp_forbid_bare_lf
+	    && (VSTRING_LEN(vp) == 0 || vstring_end(vp)[-1] != '\r'))
+	    smtp_seen_bare_lf = 1;
 	while (VSTRING_LEN(vp) > 0 && vstring_end(vp)[-1] == '\r')
 	    vstring_truncate(vp, VSTRING_LEN(vp) - 1);
 	VSTRING_TERMINATE(vp);
